@@ -42,7 +42,6 @@
 #include "tile.h"
 #include "tilelayer.h"
 #include "tilesetmanager.h"
-#include "terrain.h"
 #include "wangset.h"
 
 #include <QCoreApplication>
@@ -54,6 +53,8 @@
 
 #include <memory>
 
+#include "qtcompat_p.h"
+
 using namespace Tiled;
 using namespace Tiled::Internal;
 
@@ -90,6 +91,7 @@ private:
     void readTilesetEditorSettings(Tileset &tileset);
     void readTilesetTile(Tileset &tileset);
     void readTilesetGrid(Tileset &tileset);
+    void readTilesetTransformations(Tileset &tileset);
     void readTilesetImage(Tileset &tileset);
     void readTilesetTerrainTypes(Tileset &tileset);
     void readTilesetWangSets(Tileset &tileset);
@@ -242,19 +244,14 @@ std::unique_ptr<Map> MapReaderPrivate::readMap()
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("map"));
 
     const QXmlStreamAttributes atts = xml.attributes();
-    const int mapWidth = atts.value(QLatin1String("width")).toInt();
-    const int mapHeight = atts.value(QLatin1String("height")).toInt();
-    const int tileWidth = atts.value(QLatin1String("tilewidth")).toInt();
-    const int tileHeight = atts.value(QLatin1String("tileheight")).toInt();
-    const int infinite = atts.value(QLatin1String("infinite")).toInt();
-    const int hexSideLength = atts.value(QLatin1String("hexsidelength")).toInt();
 
     const QString orientationString =
             atts.value(QLatin1String("orientation")).toString();
-    const Map::Orientation orientation =
-            orientationFromString(orientationString);
 
-    if (orientation == Map::Unknown) {
+    Map::Parameters mapParameters;
+    mapParameters.orientation = orientationFromString(orientationString);
+
+    if (mapParameters.orientation == Map::Unknown) {
         xml.raiseError(tr("Unsupported map orientation: \"%1\"")
                        .arg(orientationString));
     }
@@ -263,17 +260,27 @@ std::unique_ptr<Map> MapReaderPrivate::readMap()
     const QString staggerIndex = atts.value(QLatin1String("staggerindex")).toString();
     const QString renderOrder = atts.value(QLatin1String("renderorder")).toString();
 
+    mapParameters.renderOrder = renderOrderFromString(renderOrder);
+    mapParameters.width = atts.value(QLatin1String("width")).toInt();
+    mapParameters.height = atts.value(QLatin1String("height")).toInt();
+    mapParameters.tileWidth = atts.value(QLatin1String("tilewidth")).toInt();
+    mapParameters.tileHeight = atts.value(QLatin1String("tileheight")).toInt();
+    mapParameters.infinite = atts.value(QLatin1String("infinite")).toInt();
+    mapParameters.hexSideLength = atts.value(QLatin1String("hexsidelength")).toInt();
+    mapParameters.staggerAxis = staggerAxisFromString(staggerAxis);
+    mapParameters.staggerIndex = staggerIndexFromString(staggerIndex);
+
+    const QString backgroundColor = atts.value(QLatin1String("backgroundcolor")).toString();
+    if (QColor::isValidColor(backgroundColor))
+        mapParameters.backgroundColor = QColor(backgroundColor);
+
+    mMap = std::make_unique<Map>(mapParameters);
+
     bool compressionLevelOk;
     const int compressionLevel = atts.value(QLatin1String("compressionlevel")).toInt(&compressionLevelOk);
-
     const int nextLayerId = atts.value(QLatin1String("nextlayerid")).toInt();
     const int nextObjectId = atts.value(QLatin1String("nextobjectid")).toInt();
 
-    mMap = std::make_unique<Map>(orientation, mapWidth, mapHeight, tileWidth, tileHeight, infinite);
-    mMap->setHexSideLength(hexSideLength);
-    mMap->setStaggerAxis(staggerAxisFromString(staggerAxis));
-    mMap->setStaggerIndex(staggerIndexFromString(staggerIndex));
-    mMap->setRenderOrder(renderOrderFromString(renderOrder));
     if (compressionLevelOk)
         mMap->setCompressionLevel(compressionLevel);
     if (nextLayerId)
@@ -281,10 +288,6 @@ std::unique_ptr<Map> MapReaderPrivate::readMap()
     if (nextObjectId)
         mMap->setNextObjectId(nextObjectId);
 
-    const QString backgroundColor = atts.value(QLatin1String("backgroundcolor")).toString();
-    if (QColor::isValidColor(backgroundColor))
-        mMap->setBackgroundColor(QColor(backgroundColor));
-
     while (xml.readNextStartElement()) {
         if (xml.name() == QLatin1String("editorsettings"))
             readMapEditorSettings(*mMap);
@@ -349,7 +352,9 @@ void MapReaderPrivate::readMapEditorSettings(Map &map)
         } else if (xml.name() == QLatin1String("export")) {
             const QXmlStreamAttributes atts = xml.attributes();
 
-            map.exportFileName = QDir::cleanPath(mPath.filePath(atts.value(QLatin1String("target")).toString()));
+            const QString target = atts.value(QLatin1String("target")).toString();
+            if (!target.isEmpty() && target != QLatin1String("."))
+                map.exportFileName = QDir::cleanPath(mPath.filePath(target));
             map.exportFormat = atts.value(QLatin1String("format")).toString();
 
             xml.skipCurrentElement();
@@ -374,58 +379,62 @@ SharedTileset MapReaderPrivate::readTileset()
         const QString name = atts.value(QLatin1String("name")).toString();
         const int tileWidth = atts.value(QLatin1String("tilewidth")).toInt();
         const int tileHeight = atts.value(QLatin1String("tileheight")).toInt();
+
+        if (tileWidth < 0 || tileHeight < 0
+            || (firstGid == 0 && !mReadingExternalTileset)) {
+            xml.raiseError(tr("Invalid tileset parameters for tileset"
+                              " '%1'").arg(name));
+            return {};
+        }
+
         const int tileSpacing = atts.value(QLatin1String("spacing")).toInt();
         const int margin = atts.value(QLatin1String("margin")).toInt();
         const int columns = atts.value(QLatin1String("columns")).toInt();
         const QString backgroundColor = atts.value(QLatin1String("backgroundcolor")).toString();
         const QString alignment = atts.value(QLatin1String("objectalignment")).toString();
 
-        if (tileWidth < 0 || tileHeight < 0
-            || (firstGid == 0 && !mReadingExternalTileset)) {
-            xml.raiseError(tr("Invalid tileset parameters for tileset"
-                              " '%1'").arg(name));
-        } else {
-            tileset = Tileset::create(name, tileWidth, tileHeight,
-                                      tileSpacing, margin);
+        tileset = Tileset::create(name, tileWidth, tileHeight,
+                                  tileSpacing, margin);
 
-            tileset->setColumnCount(columns);
+        tileset->setColumnCount(columns);
 
-            if (QColor::isValidColor(backgroundColor))
-                tileset->setBackgroundColor(QColor(backgroundColor));
+        if (QColor::isValidColor(backgroundColor))
+            tileset->setBackgroundColor(QColor(backgroundColor));
 
-            tileset->setObjectAlignment(alignmentFromString(alignment));
+        tileset->setObjectAlignment(alignmentFromString(alignment));
 
-            while (xml.readNextStartElement()) {
-                if (xml.name() == QLatin1String("editorsettings")) {
-                    readTilesetEditorSettings(*tileset);
-                } else if (xml.name() == QLatin1String("tile")) {
-                    readTilesetTile(*tileset);
-                } else if (xml.name() == QLatin1String("tileoffset")) {
-                    const QXmlStreamAttributes oa = xml.attributes();
-                    int x = oa.value(QLatin1String("x")).toInt();
-                    int y = oa.value(QLatin1String("y")).toInt();
-                    tileset->setTileOffset(QPoint(x, y));
-                    xml.skipCurrentElement();
-                } else if (xml.name() == QLatin1String("grid")) {
-                    readTilesetGrid(*tileset);
-                } else if (xml.name() == QLatin1String("properties")) {
-                    tileset->mergeProperties(readProperties());
-                } else if (xml.name() == QLatin1String("image")) {
-                    if (tileWidth == 0 || tileHeight == 0) {
-                        xml.raiseError(tr("Invalid tileset parameters for tileset"
-                                          " '%1'").arg(name));
-                        tileset.clear();
-                        break;
-                    } else {
-                        readTilesetImage(*tileset);
-                    }
-                } else if (xml.name() == QLatin1String("terraintypes")) {
-                    readTilesetTerrainTypes(*tileset);
-                } else if (xml.name() == QLatin1String("wangsets")) {
-                    readTilesetWangSets(*tileset);
+        while (xml.readNextStartElement()) {
+            if (xml.name() == QLatin1String("editorsettings")) {
+                readTilesetEditorSettings(*tileset);
+            } else if (xml.name() == QLatin1String("tile")) {
+                readTilesetTile(*tileset);
+            } else if (xml.name() == QLatin1String("tileoffset")) {
+                const QXmlStreamAttributes oa = xml.attributes();
+                int x = oa.value(QLatin1String("x")).toInt();
+                int y = oa.value(QLatin1String("y")).toInt();
+                tileset->setTileOffset(QPoint(x, y));
+                xml.skipCurrentElement();
+            } else if (xml.name() == QLatin1String("grid")) {
+                readTilesetGrid(*tileset);
+            } else if (xml.name() == QLatin1String("transformations")) {
+                readTilesetTransformations(*tileset);
+            } else if (xml.name() == QLatin1String("properties")) {
+                tileset->mergeProperties(readProperties());
+            } else if (xml.name() == QLatin1String("image")) {
+                if (tileWidth == 0 || tileHeight == 0) {
+                    xml.raiseError(tr("Invalid tileset parameters for tileset"
+                                      " '%1'").arg(name));
+                    tileset.clear();
+                    break;
                 } else {
-                    readUnknownElement();
+                    readTilesetImage(*tileset);
                 }
+            } else if (xml.name() == QLatin1String("terraintypes")) {
+                readTilesetTerrainTypes(*tileset);
+            } else if (xml.name() == QLatin1String("wangsets")) {
+                readTilesetWangSets(*tileset);
+            } else {
+                readUnknownElement();
             }
         }
     } else { // External tileset
@@ -457,7 +466,9 @@ void MapReaderPrivate::readTilesetEditorSettings(Tileset &tileset)
         if (xml.name() == QLatin1String("export")) {
             const QXmlStreamAttributes atts = xml.attributes();
 
-            tileset.exportFileName = QDir::cleanPath(mPath.filePath(atts.value(QLatin1String("target")).toString()));
+            const QString target = atts.value(QLatin1String("target")).toString();
+            if (!target.isEmpty() && target != QLatin1String("."))
+                tileset.exportFileName = QDir::cleanPath(mPath.filePath(target));
             tileset.exportFormat = atts.value(QLatin1String("format")).toString();
 
             xml.skipCurrentElement();
@@ -483,20 +494,30 @@ void MapReaderPrivate::readTilesetTile(Tileset &tileset)
 
     tile->setType(atts.value(QLatin1String("type")).toString());
 
-    // Read tile quadrant terrain ids
-    QString terrain = atts.value(QLatin1String("terrain")).toString();
-    if (!terrain.isEmpty()) {
-        QStringList quadrants = terrain.split(QLatin1String(","));
+    // Read tile quadrant terrain ids as Wang IDs. This is possible because the
+    // terrain types (loaded as WangSet) are always stored before the tiles.
+    const QStringRef terrain = atts.value(QLatin1String("terrain"));
+    if (!terrain.isEmpty() && tileset.wangSetCount() > 0) {
+        QVector<QStringRef> quadrants = terrain.split(QLatin1Char(','));
+        WangId wangId;
         if (quadrants.size() == 4) {
             for (int i = 0; i < 4; ++i) {
-                int t = quadrants[i].isEmpty() ? -1 : quadrants[i].toInt();
-                tile->setCornerTerrainId(i, t);
+                int c = quadrants[i].isEmpty() ? 0 : quadrants[i].toInt() + 1;
+                switch (i) {
+                case 0: wangId.setIndexColor(WangId::TopLeft, c); break;
+                case 1: wangId.setIndexColor(WangId::TopRight, c); break;
+                case 2: wangId.setIndexColor(WangId::BottomLeft, c); break;
+                case 3: wangId.setIndexColor(WangId::BottomRight, c); break;
+                }
             }
         }
+
+        if (wangId)
+            tileset.wangSet(0)->setWangId(id, wangId);
     }
 
     // Read tile probability
-    QStringRef probability = atts.value(QLatin1String("probability"));
+    const auto probability = atts.value(QLatin1String("probability"));
     if (!probability.isEmpty())
         tile->setProbability(probability.toDouble());
 
@@ -577,6 +598,27 @@ void MapReaderPrivate::readTilesetGrid(Tileset &tileset)
     xml.skipCurrentElement();
 }
 
+void MapReaderPrivate::readTilesetTransformations(Tileset &tileset)
+{
+    Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("transformations"));
+
+    const QXmlStreamAttributes atts = xml.attributes();
+
+    Tileset::TransformationFlags transformations;
+    if (atts.value(QLatin1String("hflip")).toInt())
+        transformations |= Tileset::AllowFlipHorizontally;
+    if (atts.value(QLatin1String("vflip")).toInt())
+        transformations |= Tileset::AllowFlipVertically;
+    if (atts.value(QLatin1String("rotate")).toInt())
+        transformations |= Tileset::AllowRotate;
+    if (atts.value(QLatin1String("preferuntransformed")).toInt())
+        transformations |= Tileset::PreferUntransformed;
+
+    tileset.setTransformationFlags(transformations);
+
+    xml.skipCurrentElement();
+}
+
 void MapReaderPrivate::readTilesetImage(Tileset &tileset)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("image"));
@@ -609,7 +651,7 @@ ImageReference MapReaderPrivate::readImage()
         while (xml.readNextStartElement()) {
             if (xml.name() == QLatin1String("data")) {
                 const QXmlStreamAttributes atts = xml.attributes();
-                QStringRef encoding = atts.value(QLatin1String("encoding"));
+                const auto encoding = atts.value(QLatin1String("encoding"));
 
                 image.data = xml.readElementText().toLatin1();
                 if (encoding == QLatin1String("base64"))
@@ -664,17 +706,22 @@ void MapReaderPrivate::readTilesetTerrainTypes(Tileset &tileset)
 {
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("terraintypes"));
 
+    auto wangSet = std::make_unique<WangSet>(&tileset, tr("Terrains"), WangSet::Corner, -1);
+    int colorCount = 0;
+
     while (xml.readNextStartElement()) {
         if (xml.name() == QLatin1String("terrain")) {
-            const QXmlStreamAttributes atts = xml.attributes();
-            QString name = atts.value(QLatin1String("name")).toString();
-            int tile = atts.value(QLatin1String("tile")).toInt();
+            wangSet->setColorCount(++colorCount);
+            const auto &wc = wangSet->colorAt(colorCount);
 
-            Terrain *terrain = tileset.addTerrain(name, tile);
+            const QXmlStreamAttributes atts = xml.attributes();
+
+            wc->setName(atts.value(QLatin1String("name")).toString());
+            wc->setImageId(atts.value(QLatin1String("tile")).toInt());
 
             while (xml.readNextStartElement()) {
                 if (xml.name() == QLatin1String("properties"))
-                    terrain->mergeProperties(readProperties());
+                    wc->mergeProperties(readProperties());
                 else
                     readUnknownElement();
             }
@@ -682,6 +729,9 @@ void MapReaderPrivate::readTilesetTerrainTypes(Tileset &tileset)
             readUnknownElement();
         }
     }
+
+    if (wangSet->colorCount() > 0)
+        tileset.addWangSet(std::move(wangSet));
 }
 
 void MapReaderPrivate::readTilesetWangSets(Tileset &tileset)
@@ -691,61 +741,96 @@ void MapReaderPrivate::readTilesetWangSets(Tileset &tileset)
     while (xml.readNextStartElement()) {
         if (xml.name() == QLatin1String("wangset")) {
             const QXmlStreamAttributes atts = xml.attributes();
-            QString name = atts.value(QLatin1String("name")).toString();
-            int tile = atts.value(QLatin1String("tile")).toInt();
+            const QString name = atts.value(QLatin1String("name")).toString();
+            const WangSet::Type type = wangSetTypeFromString(atts.value(QLatin1String("type")).toString());
+            const int tileId = atts.value(QLatin1String("tile")).toInt();
 
-            auto wangSet = std::make_unique<WangSet>(&tileset, name, tile);
+            auto wangSet = std::make_unique<WangSet>(&tileset, name, type, tileId);
+
+            // For backwards-compatibility
+            QVector<int> cornerColors;
+            QVector<int> edgeColors;
 
             while (xml.readNextStartElement()) {
+                const bool isCorner = xml.name() == QLatin1String("wangcornercolor");
+                const bool isEdge = xml.name() == QLatin1String("wangedgecolor");
+
                 if (xml.name() == QLatin1String("properties")) {
                     wangSet->mergeProperties(readProperties());
                 } else if (xml.name() == QLatin1String("wangtile")) {
                     const QXmlStreamAttributes tileAtts = xml.attributes();
-                    int tileId = tileAtts.value(QLatin1String("tileid")).toInt();
-                    WangId wangId = tileAtts.value(QLatin1String("wangid")).toUInt(nullptr, 16);
+                    const int tileId = tileAtts.value(QLatin1String("tileid")).toInt();
+                    const QStringRef wangIdString = tileAtts.value(QLatin1String("wangid"));
 
-                    if (!wangSet->wangIdIsValid(wangId)) {
-                        xml.raiseError(QLatin1String("Invalid wangId given for tileId: ") + QString::number(tileId));
+                    bool ok = true;
+                    WangId wangId;
+                    if (wangIdString.contains(QLatin1Char(',')))
+                        wangId = WangId::fromString(wangIdString, &ok);
+                    else
+                        wangId = WangId::fromUint(wangIdString.toUInt(nullptr, 16));
+
+                    // Backwards compatibility with TMX 1.4:
+                    // If the wang set was using explicit corner and edge colors,
+                    // map the WangId to the unified colors.
+                    if (!cornerColors.isEmpty() || !edgeColors.isEmpty()) {
+                        for (int i = 0; i < WangId::NumCorners; ++i) {
+                            int color = wangId.cornerColor(i);
+                            if (color > 0 && color <= cornerColors.size())
+                                wangId.setCornerColor(i, cornerColors.at(color - 1));
+                        }
+                        for (int i = 0; i < WangId::NumEdges; ++i) {
+                            int color = wangId.edgeColor(i);
+                            if (color > 0 && color <= edgeColors.size())
+                                wangId.setEdgeColor(i, edgeColors.at(color - 1));
+                        }
+                    }
+
+                    if (!wangSet->wangIdIsValid(wangId) || !ok) {
+                        xml.raiseError(QStringLiteral("Invalid wangId \"%1\" given for tileId %2").arg(wangIdString.toString(),
+                                                                                                       QString::number(tileId)));
                         return;
                     }
 
-                    bool fH = tileAtts.value(QLatin1String("hflip")).toInt();
-                    bool fV = tileAtts.value(QLatin1String("vflip")).toInt();
-                    bool fA = tileAtts.value(QLatin1String("dflip")).toInt();
-
-                    Tile *tile = tileset.findOrCreateTile(tileId);
-
-                    WangTile wangTile(tile, wangId);
-                    wangTile.setFlippedHorizontally(fH);
-                    wangTile.setFlippedVertically(fV);
-                    wangTile.setFlippedAntiDiagonally(fA);
-
-                    wangSet->addWangTile(wangTile);
+                    wangSet->setWangId(tileId, wangId);
 
                     xml.skipCurrentElement();
-                } else if (xml.name() == QLatin1String("wangedgecolor")
-                           || xml.name() == QLatin1String("wangcornercolor")) {
+                } else if (xml.name() == QLatin1String("wangcolor") || isCorner || isEdge) {
                     const QXmlStreamAttributes wangColorAtts = xml.attributes();
-                    bool isEdge = xml.name() == QLatin1String("wangedgecolor");
-                    QString name = wangColorAtts.value(QLatin1String("name")).toString();
-                    QColor color = wangColorAtts.value(QLatin1String("color")).toString();
-                    int imageId = wangColorAtts.value(QLatin1String("tile")).toInt();
-                    qreal probability = wangColorAtts.value(QLatin1String("probability")).toDouble();
+                    const QString name = wangColorAtts.value(QLatin1String("name")).toString();
+                    const QColor color = wangColorAtts.value(QLatin1String("color")).toString();
+                    const int imageId = wangColorAtts.value(QLatin1String("tile")).toInt();
+                    const qreal probability = wangColorAtts.value(QLatin1String("probability")).toDouble();
 
                     auto wc = QSharedPointer<WangColor>::create(0,
-                                                                isEdge,
                                                                 name,
                                                                 color,
                                                                 imageId,
                                                                 probability);
+
+                    while (xml.readNextStartElement()) {
+                        if (xml.name() == QLatin1String("properties"))
+                            wc->mergeProperties(readProperties());
+                        else
+                            readUnknownElement();
+                    }
+
                     wangSet->addWangColor(wc);
 
-                    xml.skipCurrentElement();
+                    if (isCorner)
+                        cornerColors.append(wc->colorIndex());
+                    if (isEdge)
+                        edgeColors.append(wc->colorIndex());
                 } else {
                     readUnknownElement();
                 }
             }
 
+            // Do something useful if we loaded an old Wang set
+            if (cornerColors.isEmpty() && !edgeColors.isEmpty())
+                wangSet->setType(WangSet::Edge);
+            if (edgeColors.isEmpty() && !cornerColors.isEmpty())
+                wangSet->setType(WangSet::Corner);
+
             tileset.addWangSet(std::move(wangSet));
         } else {
             readUnknownElement();
@@ -756,10 +841,10 @@ void MapReaderPrivate::readTilesetWangSets(Tileset &tileset)
 static void readLayerAttributes(Layer &layer,
                                 const QXmlStreamAttributes &atts)
 {
-    const QStringRef idRef = atts.value(QLatin1String("id"));
-    const QStringRef opacityRef = atts.value(QLatin1String("opacity"));
-    const QStringRef visibleRef = atts.value(QLatin1String("visible"));
-    const QStringRef lockedRef = atts.value(QLatin1String("locked"));
+    const auto idRef = atts.value(QLatin1String("id"));
+    const auto opacityRef = atts.value(QLatin1String("opacity"));
+    const auto visibleRef = atts.value(QLatin1String("visible"));
+    const auto lockedRef = atts.value(QLatin1String("locked"));
 
     bool ok;
     const int id = idRef.toInt(&ok);
@@ -770,7 +855,7 @@ static void readLayerAttributes(Layer &layer,
     if (ok)
         layer.setOpacity(opacity);
 
-    const QStringRef tintColor = atts.value(QLatin1String("tintcolor"));
+    const auto tintColor = atts.value(QLatin1String("tintcolor"));
     if (!tintColor.isEmpty())
         layer.setTintColor(QColor(tintColor.toString()));
 
@@ -786,6 +871,16 @@ static void readLayerAttributes(Layer &layer,
                          atts.value(QLatin1String("offsety")).toDouble());
 
     layer.setOffset(offset);
+
+    QPointF parallaxFactor(1.0, 1.0);
+    const qreal factorX = atts.value(QLatin1String("parallaxx")).toDouble(&ok);
+    if (ok)
+        parallaxFactor.setX(factorX);
+    const qreal factorY = atts.value(QLatin1String("parallaxy")).toDouble(&ok);
+    if (ok)
+        parallaxFactor.setY(factorY);
+
+    layer.setParallaxFactor(parallaxFactor);
 }
 
 std::unique_ptr<TileLayer> MapReaderPrivate::readTileLayer()
@@ -819,8 +914,8 @@ void MapReaderPrivate::readTileLayerData(TileLayer &tileLayer)
     Q_ASSERT(xml.isStartElement() && xml.name() == QLatin1String("data"));
 
     const QXmlStreamAttributes atts = xml.attributes();
-    QStringRef encoding = atts.value(QLatin1String("encoding"));
-    QStringRef compression = atts.value(QLatin1String("compression"));
+    const auto encoding = atts.value(QLatin1String("encoding"));
+    const auto compression = atts.value(QLatin1String("compression"));
 
     Map::LayerDataFormat layerDataFormat;
     if (encoding.isEmpty()) {
@@ -1085,7 +1180,7 @@ std::unique_ptr<MapObject> MapReaderPrivate::readObject()
     const qreal width = atts.value(QLatin1String("width")).toDouble();
     const qreal height = atts.value(QLatin1String("height")).toDouble();
     const QString type = atts.value(QLatin1String("type")).toString();
-    const QStringRef visibleRef = atts.value(QLatin1String("visible"));
+    const auto visibleRef = atts.value(QLatin1String("visible"));
 
     const QPointF pos(x, y);
     const QSizeF size(width, height);
@@ -1180,12 +1275,21 @@ QPolygonF MapReaderPrivate::readPolygon()
             break;
         }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+        const qreal x = QStringView(point).left(commaPos).toDouble(&ok);
+        if (!ok)
+            break;
+        const qreal y = QStringView(point).mid(commaPos + 1).toDouble(&ok);
+        if (!ok)
+            break;
+#else
         const qreal x = point.leftRef(commaPos).toDouble(&ok);
         if (!ok)
             break;
         const qreal y = point.midRef(commaPos + 1).toDouble(&ok);
         if (!ok)
             break;
+#endif
 
         polygon.append(QPointF(x, y));
     }
@@ -1225,13 +1329,13 @@ TextData MapReaderPrivate::readObjectText()
     textData.font.setStrikeOut(intAttribute(atts, "strikeout", 0) == 1);
     textData.font.setKerning(intAttribute(atts, "kerning", 1) == 1);
 
-    QStringRef colorString = atts.value(QLatin1String("color"));
+    const auto colorString = atts.value(QLatin1String("color"));
     if (!colorString.isEmpty())
         textData.color = QColor(colorString.toString());
 
     Qt::Alignment alignment;
 
-    QStringRef hAlignString = atts.value(QLatin1String("halign"));
+    const auto hAlignString = atts.value(QLatin1String("halign"));
     if (hAlignString == QLatin1String("center"))
         alignment |= Qt::AlignHCenter;
     else if (hAlignString == QLatin1String("right"))
@@ -1241,7 +1345,7 @@ TextData MapReaderPrivate::readObjectText()
     else
         alignment |= Qt::AlignLeft;
 
-    QStringRef vAlignString = atts.value(QLatin1String("valign"));
+    const auto vAlignString = atts.value(QLatin1String("valign"));
     if (vAlignString == QLatin1String("center"))
         alignment |= Qt::AlignVCenter;
     else if (vAlignString == QLatin1String("bottom"))
