@@ -36,7 +36,6 @@
 #include <QList>
 #include <QPixmap>
 #include <QPoint>
-#include <QPointer>
 #include <QSharedPointer>
 #include <QString>
 #include <QVector>
@@ -49,8 +48,6 @@ namespace Tiled {
 
 class Tile;
 class Tileset;
-class TilesetFormat;
-class Terrain;
 class WangSet;
 
 typedef QSharedPointer<Tileset> SharedTileset;
@@ -65,13 +62,10 @@ typedef QSharedPointer<Tileset> SharedTileset;
  */
 class TILEDSHARED_EXPORT Tileset : public Object
 {
-    Q_OBJECT
-
 public:
     /**
      * The orientation of the tileset determines the projection used in the
-     * TileCollisionDock and for the terrain information overlay of the
-     * TilesetView.
+     * TileCollisionDock and for the Wang color overlay of the TilesetView.
      */
     enum Orientation {
         Orthogonal,
@@ -115,8 +109,8 @@ public:
     void setFileName(const QString &fileName);
     bool isExternal() const;
 
-    void setFormat(TilesetFormat *format);
-    TilesetFormat *format() const;
+    void setFormat(const QString &format);
+    QString format() const;
 
     int tileWidth() const;
     int tileHeight() const;
@@ -130,6 +124,9 @@ public:
     int margin() const;
     void setMargin(int margin);
 
+    Alignment objectAlignment() const;
+    void setObjectAlignment(Alignment objectAlignment);
+
     QPoint tileOffset() const;
     void setTileOffset(QPoint offset);
 
@@ -139,9 +136,11 @@ public:
     QSize gridSize() const;
     void setGridSize(QSize gridSize);
 
-    const QMap<int, Tile*> &tiles() const;
+    const QMap<int, Tile*> &tilesById() const;
+    const QList<Tile*> &tiles() const;
     inline Tile *findTile(int id) const;
     Tile *tileAt(int id) const { return findTile(id); } // provided for Python
+    int findTileLocation(Tile *tile) const;
     Tile *findOrCreateTile(int id);
     int tileCount() const;
 
@@ -180,31 +179,21 @@ public:
     int columnCountForWidth(int width) const;
     int rowCountForHeight(int height) const;
 
-    const QList<Terrain*> &terrains() const;
-    int terrainCount() const;
-    Terrain *terrain(int index) const;
-
-    Terrain *addTerrain(const QString &name, int imageTileId);
-    void insertTerrain(int index, Terrain *terrain);
-    Terrain *takeTerrainAt(int index);
-    void swapTerrains(int index, int swapIndex);
-
-    int terrainTransitionPenalty(int terrainType0, int terrainType1) const;
-    int maximumTerrainDistance() const;
-
     const QList<WangSet*> &wangSets() const;
     int wangSetCount() const;
     WangSet *wangSet(int index) const;
 
-    void addWangSet(WangSet *wangSet);
-    void addWangSet(std::unique_ptr<WangSet> &&wangSet);
-    void insertWangSet(int index, WangSet *wangSet);
-    WangSet *takeWangSetAt(int index);
+    void addWangSet(std::unique_ptr<WangSet> wangSet);
+    void insertWangSet(int index, std::unique_ptr<WangSet> wangSet);
+    std::unique_ptr<WangSet> takeWangSetAt(int index);
 
     Tile *addTile(const QPixmap &image, const QUrl &source = QUrl());
     void addTiles(const QList<Tile*> &tiles);
     void removeTiles(const QList<Tile *> &tiles);
     void deleteTile(int id);
+    QList<int> relocateTiles(const QList<Tile *> &tiles, int location);
+
+    bool anyTileOutOfOrder() const;
 
     void setNextTileId(int nextId);
     int nextTileId() const;
@@ -214,8 +203,6 @@ public:
                       const QPixmap &image,
                       const QUrl &source = QUrl());
 
-    void markTerrainDistancesDirty();
-
     SharedTileset sharedPointer() const;
 
     void setOriginalTileset(const SharedTileset &original);
@@ -226,6 +213,18 @@ public:
     LoadingStatus status() const;
     LoadingStatus imageStatus() const;
 
+    enum TransformationFlag {
+        NoTransformation        = 0,
+        AllowFlipHorizontally   = 1 << 0,
+        AllowFlipVertically     = 1 << 1,
+        AllowRotate             = 1 << 2,
+        PreferUntransformed     = 1 << 3,
+    };
+    Q_DECLARE_FLAGS(TransformationFlags, TransformationFlag)
+
+    TransformationFlags transformationFlags() const;
+    void setTransformationFlags(TransformationFlags flags);
+
     void swap(Tileset &other);
 
     SharedTileset clone() const;
@@ -249,7 +248,6 @@ public:
 
 private:
     void updateTileSize();
-    void recalculateTerrainDistances();
 
     QString mName;
     QString mFileName;
@@ -259,20 +257,20 @@ private:
     int mTileSpacing;
     int mMargin;
     QPoint mTileOffset;
+    Alignment mObjectAlignment;
     Orientation mOrientation;
     QSize mGridSize;
     int mColumnCount;
     int mExpectedColumnCount;
     int mExpectedRowCount;
     int mNextTileId;
-    int mMaximumTerrainDistance;
-    QMap<int, Tile*> mTiles;
-    QList<Terrain*> mTerrainTypes;
+    QMap<int, Tile*> mTilesById;
+    QList<Tile*> mTiles;
     QList<WangSet*> mWangSets;
-    bool mTerrainDistancesDirty;
     LoadingStatus mStatus;
     QColor mBackgroundColor;
-    QPointer<TilesetFormat> mFormat;
+    QString mFormat;
+    TransformationFlags mTransformationFlags;
 
     QWeakPointer<Tileset> mWeakPointer;
     QWeakPointer<Tileset> mOriginalTileset;
@@ -360,6 +358,22 @@ inline int Tileset::margin() const
     return mMargin;
 }
 
+/**
+ * Returns the alignment to use for tile objects.
+ */
+inline Alignment Tileset::objectAlignment() const
+{
+    return mObjectAlignment;
+}
+
+/**
+ * @see objectAlignment
+ */
+inline void Tileset::setObjectAlignment(Alignment objectAlignment)
+{
+    mObjectAlignment = objectAlignment;
+}
+
 /**
  * Returns the offset that is applied when drawing the tiles in this
  * tileset.
@@ -410,10 +424,15 @@ inline void Tileset::setGridSize(QSize gridSize)
     mGridSize = gridSize;
 }
 
+inline const QMap<int, Tile *> &Tileset::tilesById() const
+{
+    return mTilesById;
+}
+
 /**
  * Returns a const reference to the tiles in this tileset.
  */
-inline const QMap<int, Tile *> &Tileset::tiles() const
+inline const QList<Tile*> &Tileset::tiles() const
 {
     return mTiles;
 }
@@ -424,7 +443,7 @@ inline const QMap<int, Tile *> &Tileset::tiles() const
  */
 inline Tile *Tileset::findTile(int id) const
 {
-    return mTiles.value(id);
+    return mTilesById.value(id);
 }
 
 /**
@@ -554,30 +573,6 @@ inline bool Tileset::isCollection() const
     return imageSource().isEmpty();
 }
 
-/**
- * Returns a const reference to the list of terrains in this tileset.
- */
-inline const QList<Terrain *> &Tileset::terrains() const
-{
-    return mTerrainTypes;
-}
-
-/**
- * Returns the number of terrain types in this tileset.
- */
-inline int Tileset::terrainCount() const
-{
-    return mTerrainTypes.size();
-}
-
-/**
- * Returns the terrain type at the given \a index.
- */
-inline Terrain *Tileset::terrain(int index) const
-{
-    return index >= 0 ? mTerrainTypes[index] : nullptr;
-}
-
 inline const QList<WangSet*> &Tileset::wangSets() const
 {
     return mWangSets;
@@ -618,14 +613,6 @@ inline int Tileset::takeNextTileId()
     return mNextTileId++;
 }
 
-/**
- * Used by the Tile class when its terrain information changes.
- */
-inline void Tileset::markTerrainDistancesDirty()
-{
-    mTerrainDistancesDirty = true;
-}
-
 inline SharedTileset Tileset::sharedPointer() const
 {
     return SharedTileset(mWeakPointer);
@@ -667,6 +654,19 @@ inline LoadingStatus Tileset::imageStatus() const
     return mImageReference.status;
 }
 
+inline Tileset::TransformationFlags Tileset::transformationFlags() const
+{
+    return mTransformationFlags;
+}
+
+inline void Tileset::setTransformationFlags(TransformationFlags flags)
+{
+    mTransformationFlags = flags;
+}
+
 } // namespace Tiled
 
+Q_DECLARE_METATYPE(Tiled::Tileset*)
 Q_DECLARE_METATYPE(Tiled::SharedTileset)
+
+Q_DECLARE_OPERATORS_FOR_FLAGS(Tiled::Tileset::TransformationFlags)
