@@ -110,13 +110,13 @@ public:
      * Activates this tool. If the tool plans to add any items to the scene, it
      * probably wants to do it here.
      */
-    virtual void activate(MapScene *scene) = 0;
+    virtual void activate(MapScene *scene);
 
     /**
      * Deactivates this tool. Should do any necessary cleanup to make sure the
      * tool is no longer active.
      */
-    virtual void deactivate(MapScene *scene) = 0;
+    virtual void deactivate(MapScene *scene);
 
     virtual void keyPressed(QKeyEvent *);
 
@@ -190,6 +190,7 @@ protected:
     }
 
     MapDocument *mapDocument() const { return mMapDocument; }
+    MapScene *mapScene() const { return mMapScene; }
 
     Layer *currentLayer() const;
 
@@ -223,6 +224,7 @@ private:
 
     ToolManager *mToolManager = nullptr;
     MapDocument *mMapDocument = nullptr;
+    MapScene *mMapScene = nullptr;
 };
 
 
