@@ -20,6 +20,7 @@
 
 #include "changemapproperty.h"
 
+#include "changeevents.h"
 #include "map.h"
 #include "mapdocument.h"
 #include "objectgroup.h"
@@ -30,30 +31,30 @@
 using namespace Tiled;
 
 ChangeMapProperty::ChangeMapProperty(MapDocument *mapDocument,
-                                     ChangeMapProperty::Property property,
+                                     Map::Property property,
                                      int value)
     : mMapDocument(mapDocument)
     , mProperty(property)
     , mIntValue(value)
 {
     switch (property) {
-    case TileWidth:
+    case Map::TileWidthProperty:
         setText(QCoreApplication::translate("Undo Commands",
                                             "Change Tile Width"));
         break;
-    case TileHeight:
+    case Map::TileHeightProperty:
         setText(QCoreApplication::translate("Undo Commands",
                                             "Change Tile Height"));
         break;
-    case Infinite:
+    case Map::InfiniteProperty:
         setText(QCoreApplication::translate("Undo Commands",
                                             "Change Infinite Property"));
         break;
-    case HexSideLength:
+    case Map::HexSideLengthProperty:
         setText(QCoreApplication::translate("Undo Commands",
                                             "Change Hex Side Length"));
         break;
-    case CompressionLevel:
+    case Map::CompressionLevelProperty:
         setText(QCoreApplication::translate("Undo Commands",
                                             "Change Compression Level"));
         break;
@@ -67,7 +68,7 @@ ChangeMapProperty::ChangeMapProperty(MapDocument *mapDocument,
     : QUndoCommand(QCoreApplication::translate("Undo Commands",
                                                "Change Background Color"))
     , mMapDocument(mapDocument)
-    , mProperty(BackgroundColor)
+    , mProperty(Map::BackgroundColorProperty)
     , mBackgroundColor(backgroundColor)
 {
 }
@@ -77,7 +78,7 @@ ChangeMapProperty::ChangeMapProperty(MapDocument *mapDocument,
     : QUndoCommand(QCoreApplication::translate("Undo Commands",
                                                "Change Chunk Size"))
     , mMapDocument(mapDocument)
-    , mProperty(ChunkSize)
+    , mProperty(Map::ChunkSizeProperty)
     , mChunkSize(chunkSize)
 {
 }
@@ -87,7 +88,7 @@ ChangeMapProperty::ChangeMapProperty(MapDocument *mapDocument,
     : QUndoCommand(QCoreApplication::translate("Undo Commands",
                                                "Change Stagger Axis"))
     , mMapDocument(mapDocument)
-    , mProperty(StaggerAxis)
+    , mProperty(Map::StaggerAxisProperty)
     , mStaggerAxis(staggerAxis)
 {
 }
@@ -97,7 +98,7 @@ ChangeMapProperty::ChangeMapProperty(MapDocument *mapDocument,
     : QUndoCommand(QCoreApplication::translate("Undo Commands",
                                                "Change Stagger Index"))
     , mMapDocument(mapDocument)
-    , mProperty(StaggerIndex)
+    , mProperty(Map::StaggerIndexProperty)
     , mStaggerIndex(staggerIndex)
 {
 }
@@ -107,7 +108,7 @@ ChangeMapProperty::ChangeMapProperty(MapDocument *mapDocument,
     : QUndoCommand(QCoreApplication::translate("Undo Commands",
                                                "Change Orientation"))
     , mMapDocument(mapDocument)
-    , mProperty(Orientation)
+    , mProperty(Map::OrientationProperty)
     , mOrientation(orientation)
 {
 }
@@ -117,7 +118,7 @@ ChangeMapProperty::ChangeMapProperty(MapDocument *mapDocument,
     : QUndoCommand(QCoreApplication::translate("Undo Commands",
                                                "Change Render Order"))
     , mMapDocument(mapDocument)
-    , mProperty(RenderOrder)
+    , mProperty(Map::RenderOrderProperty)
     , mRenderOrder(renderOrder)
 {
 }
@@ -127,7 +128,7 @@ ChangeMapProperty::ChangeMapProperty(MapDocument *mapDocument,
     : QUndoCommand(QCoreApplication::translate("Undo Commands",
                                                "Change Layer Data Format"))
     , mMapDocument(mapDocument)
-    , mProperty(LayerDataFormat)
+    , mProperty(Map::LayerDataFormatProperty)
     , mLayerDataFormat(layerDataFormat)
 {
 }
@@ -147,74 +148,73 @@ void ChangeMapProperty::swap()
     Map *map = mMapDocument->map();
 
     switch (mProperty) {
-    case TileWidth: {
+    case Map::TileWidthProperty: {
         const int tileWidth = map->tileWidth();
         map->setTileWidth(mIntValue);
         mIntValue = tileWidth;
         break;
     }
-    case TileHeight: {
+    case Map::TileHeightProperty: {
         const int tileHeight = map->tileHeight();
         map->setTileHeight(mIntValue);
         mIntValue = tileHeight;
         break;
     }
-    case Infinite: {
+    case Map::InfiniteProperty: {
         const int infinite = map->infinite();
         map->setInfinite(mIntValue);
         mIntValue = infinite;
         break;
     }
-    case Orientation: {
+    case Map::OrientationProperty: {
         const Map::Orientation orientation = map->orientation();
         map->setOrientation(mOrientation);
         mOrientation = orientation;
-        mMapDocument->createRenderer();
         break;
     }
-    case HexSideLength: {
+    case Map::HexSideLengthProperty: {
         const int hexSideLength = map->hexSideLength();
         map->setHexSideLength(mIntValue);
         mIntValue = hexSideLength;
         break;
     }
-    case StaggerAxis: {
+    case Map::StaggerAxisProperty: {
         const Map::StaggerAxis staggerAxis = map->staggerAxis();
         map->setStaggerAxis(mStaggerAxis);
         mStaggerAxis = staggerAxis;
         break;
     }
-    case StaggerIndex: {
+    case Map::StaggerIndexProperty: {
         const Map::StaggerIndex staggerIndex = map->staggerIndex();
         map->setStaggerIndex(mStaggerIndex);
         mStaggerIndex = staggerIndex;
         break;
     }
-    case RenderOrder: {
+    case Map::RenderOrderProperty: {
         const Map::RenderOrder renderOrder = map->renderOrder();
         map->setRenderOrder(mRenderOrder);
         mRenderOrder = renderOrder;
         break;
     }
-    case BackgroundColor: {
+    case Map::BackgroundColorProperty: {
         const QColor backgroundColor = map->backgroundColor();
         map->setBackgroundColor(mBackgroundColor);
         mBackgroundColor = backgroundColor;
         break;
     }
-    case LayerDataFormat: {
+    case Map::LayerDataFormatProperty: {
         const Map::LayerDataFormat layerDataFormat = map->layerDataFormat();
         map->setLayerDataFormat(mLayerDataFormat);
         mLayerDataFormat = layerDataFormat;
         break;
     }
-    case CompressionLevel: {
+    case Map::CompressionLevelProperty: {
         const int compressionLevel = map->compressionLevel();
         map->setCompressionLevel(mIntValue);
         mIntValue = compressionLevel;
         break;
     }
-    case ChunkSize: {
+    case Map::ChunkSizeProperty: {
         const QSize chunkSize = map->chunkSize();
         map->setChunkSize(mChunkSize);
         mChunkSize = chunkSize;
@@ -222,5 +222,6 @@ void ChangeMapProperty::swap()
     }
     }
 
+    emit mMapDocument->changed(MapChangeEvent(mProperty));
     emit mMapDocument->mapChanged();
 }
