@@ -206,6 +206,7 @@ private:
     MapRenderer *renderer() const;
 
     std::unique_ptr<Map> mDetachedMap;
+    mutable std::unique_ptr<MapRenderer> mRenderer;
     bool mReadOnly;
 
     EditableSelectedArea *mSelectedArea;
@@ -358,11 +359,6 @@ inline MapDocument *EditableMap::mapDocument() const
     return static_cast<MapDocument*>(document());
 }
 
-inline MapRenderer *EditableMap::renderer() const
-{
-    return mapDocument() ? mapDocument()->renderer() : nullptr;
-}
-
 } // namespace Tiled
 
 Q_DECLARE_METATYPE(Tiled::EditableMap*)
