@@ -255,7 +255,11 @@ QMimeData *LayerModel::mimeData(const QModelIndexList &indexes) const
 
     QMimeData *mimeData = new QMimeData;
     QByteArray encodedData;
+#if QT_VERSION < QT_VERSION_CHECK(6,0,0)
     QDataStream stream(&encodedData, QIODevice::WriteOnly);
+#else
+    QDataStream stream(&encodedData, QDataStream::WriteOnly);
+#endif
     QVector<Layer*> layers;
 
     for (const QModelIndex &index : indexes) {
@@ -294,8 +298,8 @@ bool LayerModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
 
     GroupLayer *groupLayer = static_cast<GroupLayer*>(parentLayer);
 
-    QByteArray encodedData = data->data(QLatin1String(LAYERS_MIMETYPE));
-    QDataStream stream(&encodedData, QIODevice::ReadOnly);
+    const QByteArray encodedData = data->data(QLatin1String(LAYERS_MIMETYPE));
+    QDataStream stream(encodedData);
     QList<Layer*> layers;
 
     while (!stream.atEnd()) {
@@ -613,3 +617,5 @@ void LayerModel::documentChanged(const ChangeEvent &change)
         break;
     }
 }
+
+#include "moc_layermodel.cpp"
