@@ -25,6 +25,7 @@
 
 #include "mapdocument.h"
 #include "mapitem.h"
+#include "session.h"
 
 #include <QColor>
 #include <QGraphicsScene>
@@ -40,6 +41,7 @@ class TileLayer;
 class Tileset;
 
 class AbstractTool;
+class DebugDrawItem;
 class LayerItem;
 class MapDocument;
 class MapObjectItem;
@@ -61,6 +63,7 @@ public:
     void setMapDocument(MapDocument *map);
 
     void setShowTileCollisionShapes(bool enabled);
+    void setParallaxEnabled(bool enabled);
 
     QRectF mapBoundingRect() const;
 
@@ -68,11 +71,23 @@ public:
 
     MapItem *mapItem(MapDocument *mapDocument) const;
 
+    DebugDrawItem *debugDrawItem() const;
+
+    const QRectF &viewRect() const;
+    void setViewRect(const QRectF &rect);
+
+    QPointF absolutePositionForLayer(const Layer &layer) const;
+    QPointF parallaxOffset(const Layer &layer) const;
+
+    static SessionOption<bool> enableWorlds;
+
 signals:
     void mapDocumentChanged(MapDocument *mapDocument);
 
     void sceneRefreshed();
 
+    void parallaxParametersChanged();
+
 protected:
     bool event(QEvent *event) override;
 
@@ -98,6 +113,8 @@ private:
     void updateDefaultBackgroundColor();
     void updateSceneRect();
 
+    void setWorldsEnabled(bool enabled);
+
     MapItem *takeOrCreateMapItem(const MapDocumentPtr &mapDocument,
                                  MapItem::DisplayMode displayMode);
 
@@ -106,10 +123,15 @@ private:
     MapDocument *mMapDocument = nullptr;
     QHash<MapDocument*, MapItem*> mMapItems;
     AbstractTool *mSelectedTool = nullptr;
+    DebugDrawItem *mDebugDrawItem = nullptr;
     bool mUnderMouse = false;
     bool mShowTileCollisionShapes = false;
+    bool mParallaxEnabled = true;
+    bool mWorldsEnabled = true;
+    Session::CallbackIterator mEnableWorldsCallback;
     Qt::KeyboardModifiers mCurrentModifiers = Qt::NoModifier;
     QPointF mLastMousePos;
+    QRectF mViewRect;
     QColor mDefaultBackgroundColor;
 };
 
@@ -129,4 +151,14 @@ inline MapItem *MapScene::mapItem(MapDocument *mapDocument) const
     return mMapItems.value(mapDocument);
 }
 
+inline DebugDrawItem *MapScene::debugDrawItem() const
+{
+    return mDebugDrawItem;
+}
+
+inline const QRectF &MapScene::viewRect() const
+{
+    return mViewRect;
+}
+
 } // namespace Tiled
