@@ -21,13 +21,10 @@
 #include "newmapdialog.h"
 #include "ui_newmapdialog.h"
 
-#include "hexagonalrenderer.h"
-#include "isometricrenderer.h"
 #include "map.h"
 #include "mapdocument.h"
-#include "orthogonalrenderer.h"
+#include "maprenderer.h"
 #include "session.h"
-#include "staggeredrenderer.h"
 #include "tilelayer.h"
 #include "utils.h"
 
@@ -153,10 +150,15 @@ MapDocumentPtr NewMapDialog::createMap()
     session::mapTileWidth = mUi->tileWidth->value();
     session::mapTileHeight = mUi->tileHeight->value();
 
-    std::unique_ptr<Map> map { new Map(session::mapOrientation,
-                                       session::mapWidth, session::mapHeight,
-                                       session::mapTileWidth, session::mapTileHeight,
-                                       !session::fixedSize) };
+    Map::Parameters mapParameters;
+    mapParameters.orientation = session::mapOrientation;
+    mapParameters.width = session::mapWidth;
+    mapParameters.height = session::mapHeight;
+    mapParameters.tileWidth = session::mapTileWidth;
+    mapParameters.tileHeight = session::mapTileHeight;
+    mapParameters.infinite = !session::fixedSize;
+
+    auto map = std::make_unique<Map>(mapParameters);
 
     map->setLayerDataFormat(session::layerDataFormat);
     map->setRenderOrder(session::renderOrder);
@@ -190,28 +192,15 @@ MapDocumentPtr NewMapDialog::createMap()
 
 void NewMapDialog::refreshPixelSize()
 {
-    const Map map(comboBoxValue<Map::Orientation>(mUi->orientation),
-                  mUi->mapWidth->value(),
-                  mUi->mapHeight->value(),
-                  mUi->tileWidth->value(),
-                  mUi->tileHeight->value());
+    Map::Parameters mapParameters;
+    mapParameters.orientation = comboBoxValue<Map::Orientation>(mUi->orientation);
+    mapParameters.width = mUi->mapWidth->value();
+    mapParameters.height = mUi->mapHeight->value();
+    mapParameters.tileWidth = mUi->tileWidth->value();
+    mapParameters.tileHeight = mUi->tileHeight->value();
 
-    QSize size;
-
-    switch (map.orientation()) {
-    case Map::Isometric:
-        size = IsometricRenderer(&map).mapBoundingRect().size();
-        break;
-    case Map::Staggered:
-        size = StaggeredRenderer(&map).mapBoundingRect().size();
-        break;
-    case Map::Hexagonal:
-        size = HexagonalRenderer(&map).mapBoundingRect().size();
-        break;
-    default:
-        size = OrthogonalRenderer(&map).mapBoundingRect().size();
-        break;
-    }
+    const Map map(mapParameters);
+    const QSize size = MapRenderer::create(&map)->mapBoundingRect().size();
 
     mUi->pixelSizeLabel->setText(tr("%1 x %2 pixels")
                                  .arg(size.width())
@@ -226,3 +215,5 @@ void NewMapDialog::updateWidgets(bool checked)
     mUi->heightLabel->setEnabled(checked);
     mUi->widthLabel->setEnabled(checked);
 }
+
+#include "moc_newmapdialog.cpp"
