@@ -82,7 +82,11 @@ static void findFiles(const FolderEntry &entry, int offset, const QStringList &w
 {
     for (const auto &childEntry : entry.entries) {
         if (childEntry->entries.empty()) {
-            const QStringRef relativePath = childEntry->filePath.midRef(offset);
+#if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
+            const auto relativePath = QStringView(childEntry->filePath).mid(offset);
+#else
+            const auto relativePath = childEntry->filePath.midRef(offset);
+#endif
             const int totalScore = Utils::matchingScore(words, relativePath);
 
             if (totalScore > 0) {
@@ -130,7 +134,9 @@ ProjectModel::ProjectModel(QObject *parent)
 ProjectModel::~ProjectModel()
 {
     mFoldersPendingScan.clear();
+#ifndef Q_OS_WASM
     mScanningThread.requestInterruption();
+#endif
     mScanningThread.quit();
     mScanningThread.wait();
 }
@@ -165,6 +171,7 @@ void ProjectModel::addFolder(const QString &folder)
 
     mProject.addFolder(folder);
     mFolders.push_back(std::make_unique<FolderEntry>(folder));
+    mWatcher.addPath(folder);
     scheduleFolderScan(folder);
 
     endInsertRows();
@@ -176,9 +183,14 @@ void ProjectModel::removeFolder(int row)
 {
     const QString folder = mFolders.at(row)->filePath;
 
+    QStringList watchedFilePaths;
+    watchedFilePaths.append(folder);
+    collectDirectories(*mFolders.at(row), watchedFilePaths);
+
     beginRemoveRows(QModelIndex(), row, row);
     mProject.removeFolder(row);
     mFolders.erase(mFolders.begin() + row);
+    mWatcher.removePaths(watchedFilePaths);
     endRemoveRows();
 
     emit folderRemoved(folder);
@@ -465,8 +477,10 @@ void FolderScanner::scanFolder(const QString &folder)
 
 void FolderScanner::scan(FolderEntry &folder, QSet<QString> &visitedFolders) const
 {
+#ifndef Q_OS_WASM
     if (QThread::currentThread()->isInterruptionRequested())
         return;
+#endif
 
     constexpr QDir::SortFlags sortFlags { QDir::Name | QDir::LocaleAware | QDir::DirsFirst };
     constexpr QDir::Filters filters { QDir::AllDirs | QDir::Files | QDir::NoDotAndDotDot };
@@ -496,3 +510,4 @@ void FolderScanner::scan(FolderEntry &folder, QSet<QString> &visitedFolders) con
 } // namespace Tiled
 
 #include "projectmodel.moc"
+#include "moc_projectmodel.cpp"
