@@ -81,7 +81,8 @@ bool VariantPropertyManager::isPropertyTypeSupported(int propertyType) const
     if (propertyType == filePathTypeId()
             || propertyType == displayObjectRefTypeId()
             || propertyType == tilesetParametersTypeId()
-            || propertyType == alignmentTypeId())
+            || propertyType == alignmentTypeId()
+            || propertyType == unstyledGroupTypeId())
         return true;
     return QtVariantPropertyManager::isPropertyTypeSupported(propertyType);
 }
@@ -96,6 +97,8 @@ int VariantPropertyManager::valueType(int propertyType) const
         return qMetaTypeId<TilesetDocument*>();
     if (propertyType == alignmentTypeId())
         return propertyType;
+    if (propertyType == unstyledGroupTypeId())
+        return propertyType;
     return QtVariantPropertyManager::valueType(propertyType);
 }
 
@@ -115,9 +118,9 @@ int VariantPropertyManager::attributeType(int propertyType,
 {
     if (propertyType == filePathTypeId()) {
         if (attribute == mFilterAttribute)
-            return QVariant::String;
+            return QMetaType::QString;
         if (attribute == mDirectoryAttribute)
-            return QVariant::Bool;
+            return QMetaType::Bool;
         return 0;
     }
     return QtVariantPropertyManager::attributeType(propertyType, attribute);
@@ -158,6 +161,11 @@ int VariantPropertyManager::displayObjectRefTypeId()
     return qMetaTypeId<DisplayObjectRef>();
 }
 
+int VariantPropertyManager::unstyledGroupTypeId()
+{
+    return qMetaTypeId<UnstyledGroup>();
+}
+
 QString VariantPropertyManager::objectRefLabel(const MapObject *object) const
 {
     QString label = tr("%1: ").arg(object->id());
@@ -306,7 +314,7 @@ void VariantPropertyManager::setAttribute(QtProperty *property,
     if (mFilePathAttributes.contains(property)) {
         FilePathAttributes &attributes = mFilePathAttributes[property];
         if (attribute == mFilterAttribute) {
-            if (val.type() != QVariant::String && !val.canConvert(QVariant::String))
+            if (val.userType() != QMetaType::QString && !val.canConvert(QMetaType::QString))
                 return;
             QString filter = val.toString();
             if (attributes.filter == filter)
@@ -346,7 +354,7 @@ void VariantPropertyManager::initializeProperty(QtProperty *property)
         mValues[property] = QVariant();
         if (type == filePathTypeId())
             mFilePathAttributes[property] = FilePathAttributes();
-    } else if (type == QVariant::String) {
+    } else if (type == QMetaType::QString) {
         mStringAttributes[property] = StringAttributes();
     } else if (type == alignmentTypeId()) {
         const Qt::Alignment align = Qt::AlignLeft | Qt::AlignVCenter;
@@ -497,3 +505,5 @@ QString VariantPropertyManager::indexVToString(int idx) const
 }
 
 } // namespace Tiled
+
+#include "moc_variantpropertymanager.cpp"
